@@ -1,10 +1,11 @@
+from dataclasses import dataclass
 from typing import Optional
 
 from argo_workflows.models import IoArgoprojWorkflowV1alpha1TTLStrategy
-from pydantic import BaseModel
 
 
-class TTLStrategy(BaseModel):
+@dataclass
+class TTLStrategy:
     """TTLStrategy specification for workflows.
 
     https://argoproj.github.io/argo-workflows/fields/#ttlstrategy
@@ -27,8 +28,7 @@ class TTLStrategy(BaseModel):
     seconds_after_failure: Optional[int]
     seconds_after_success: Optional[int]
 
-    @property
-    def argo_ttl_strategy(self) -> IoArgoprojWorkflowV1alpha1TTLStrategy:
+    def build(self) -> IoArgoprojWorkflowV1alpha1TTLStrategy:
         """Constructs and returns the ttl strategy"""
         return IoArgoprojWorkflowV1alpha1TTLStrategy(
             seconds_after_completion=self.seconds_after_completion,
